unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    Image1: TImage;
    AssumptionGroupBox: TGroupBox;
    StartAssumptionButton: TButton;
    CancelAssumptionButton: TButton;
    ConfirmAssumptionButton: TButton;
    ClearAllFieldButton: TButton;
    StatusBar1: TStatusBar;
    Timer1: TTimer;
    AutoSolveButton: TButton;
    Transp: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClick(Sender: TObject);
    procedure FormDblClick(Sender: TObject);
    procedure Image1Click(Sender: TObject);
    procedure Image1DblClick(Sender: TObject);
    procedure StartAssumptionButtonClick(Sender: TObject);
    procedure CancelAssumptionButtonClick(Sender: TObject);
    procedure ConfirmAssumptionButtonClick(Sender: TObject);
    procedure ClearAllFieldButtonClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure AutoSolveButtonClick(Sender: TObject);
    procedure TranspClick(Sender: TObject);
  private
    procedure OnEditClick(Sender: TObject);
    procedure OnEditChange(Sender: TObject);
    function Cell(i:integer; j:integer) : integer;
    procedure ResetWhiteEveryWhere;
    procedure ResetMarkEveryWhere;
    procedure Init;
    procedure InitHints;
    procedure DeleteNumberFromHint(i: integer; j:integer; value:integer);
    function CheckSudokuIsSolved : boolean;
    procedure UpdateStatusBar;
    { Private declarations }
  public
    { Public declarations }
end;

type
  TMyEdit = class(TEdit)
  public
    Assumption : boolean;   {if true value in this cell is only assumption, red color}
    Mark : boolean;         {analogue of color, but used internally}
end;

var
  Form1: TForm1;
  EditsArr : array of TMyEdit;
  {array of 81 input fields}

  LabelsArr : array of TLabel;
  {array of 81 labels above input fields
  they mean the possible numbers in corresponding input fields}

  AssumptionFlag : boolean;
  {if this flag is true, the assumption (guess) started, the numbers
  are marked by red}

  InitInProgressFlag : boolean;
  {initialization of field is in progress if this flag is true}

  EmptyCellsLeft : integer;
  {number of empty cells on the field}

  StartTime : TDateTime;
  const EditWidth = 48;
  const EditHeight = 48;
  const EditXSpacing = 60;
  const EditYSpacing = 60;
  const LeftMargin = 50;
  const TopMargin = 50;
  const RightMargin = 50;
  const BottomMargin = 50;
  const EditFontHeight = 40;
  const LabelFontHeight = 12;
  const LabelWidth = 60;
  const LabelHeight = 10;

implementation

uses SolvedForm;

{$R *.DFM}

procedure TForm1.InitHints;
var
   i, j, k, i1, j1, i2, j2, i3, j3, h, possible_cell_value : integer;
   hintarr : array [0..8] of integer;
   hint : string;
   isquare, jsquare : integer;
   unmarked_fields, unmarked_field_i, unmarked_field_j : integer;
   tmptext1, tmptext2 : string;
   Number1, Number2 : integer;
   String1, String2 : string;
begin
   for i:=0 to 8 do
   begin
      for j:= 0 to 8 do
      begin
         if Cell(i,j) <> 0 then
         begin
             EditsArr[i+9*j].Hint := '';
             LabelsArr[i+9*j].Caption :=  '';
         end
         else
         begin
             //init hint array
             for h:=0 to 8 do hintarr[h]:=h+1;
             hint := '';

             //look in current row
             for i1:=0 to 8 do
             begin
                 if Cell(i1, j)<>0 then hintarr[Cell(i1, j)-1] := 0;
             end;

             //look in current column
             for j1:=0 to 8 do
             begin
                 if Cell(i, j1)<>0 then hintarr[Cell(i, j1)-1] := 0;
             end;

             //look in square 3x3 near Cell(i, j)
             isquare := (i div 3);
             jsquare := (j div 3);
             for i1 := isquare*3 to isquare*3 + 2 do
             begin
                 for j1 := jsquare*3 to jsquare*3 + 2 do
                 begin
                     if Cell(i1, j1)<>0 then hintarr[Cell(i1, j1)-1] := 0;
                 end;
             end;

             //set hint
             for h:=0 to 8 do
             begin
                 if hintarr[h] <> 0 then hint := hint + IntToStr(hintarr[h]);
             end;
             EditsArr[i+9*j].Hint := hint;
             LabelsArr[i+9*j].Caption := hint;
         end;
      end;
   end;

   {marks are analogue of colour, but used internally}
   for possible_cell_value := 1 to 9 do
   begin
       ResetMarkEveryWhere;
       for i:=0 to 8 do
       begin
          for j:= 0 to 8 do
          begin
              if Cell(i, j) <> 0 then
              begin
                 EditsArr[i+9*j].Mark := true;
              end;

              if Cell(i, j) = possible_cell_value then
              begin
                  //fill column
                  for i1 := 0 to 8 do
                  begin
                      EditsArr[i1+9*j].Mark := true;
                  end;

                  //fill row
                  for j1 := 0 to 8 do
                  begin
                      EditsArr[i+9*j1].Mark := true;
                  end;

                  //fill square 3x3 near Cell(i, j)
                  isquare := (i div 3);
                  jsquare := (j div 3);
                  for i1 := isquare*3 to isquare*3 + 2 do
                  begin
                      for j1 := jsquare*3 to jsquare*3 + 2 do
                      begin
                          EditsArr[i1+9*j1].Mark := true;
                      end;
                  end;
              end;
          end;
       end;

       {now, marks are set up}

       {check if some line have only one unmarked field}
       for i:=0 to 8 do
       begin
          unmarked_fields := 0;
          for j:= 0 to 8 do
          begin
               if EditsArr[i+j*9].Mark = false then
               begin
                   Inc(unmarked_fields);

                   {only last values i,j saved if unmarked_fields > 1}
                   unmarked_field_i := i;
                   unmarked_field_j := j;
               end;
          end;
          if unmarked_fields = 1 then
          begin
              {hurra!!!}
              LabelsArr[unmarked_field_i + 9*unmarked_field_j].Caption := IntToStr(possible_cell_value);
              EditsArr[unmarked_field_i + 9*unmarked_field_j].hint := IntToStr(possible_cell_value);
          end;
       end;

       {check if some column have only one unmarked field}
       for j:=0 to 8 do
       begin
          unmarked_fields := 0;
          for i:= 0 to 8 do
          begin
               if EditsArr[i+j*9].Mark = false then
               begin
                   Inc(unmarked_fields);

                   {only last values i,j saved if unmarked_fields > 1}
                   unmarked_field_i := i;
                   unmarked_field_j := j;
               end;
          end;
          if unmarked_fields = 1 then
          begin
              {hurra!!!}
              LabelsArr[unmarked_field_i + 9*unmarked_field_j].Caption := IntToStr(possible_cell_value);
              EditsArr[unmarked_field_i + 9*unmarked_field_j].hint := IntToStr(possible_cell_value);
          end;
       end;

   end; {possible_cell_value}

   {search for 2 equal hints in one line, eg 12 and 12
   it means that in this line other cells can not have 1 and 2}
   for i:=0 to 8 do
   begin
       for j:=0 to 8 do
       begin
           for k:=j to 8 do
           begin
               if j<>k then
               begin
                   tmptext1 :=  LabelsArr[i+j*9].Caption;
                   tmptext2 := LabelsArr[i+k*9].Caption;
                   if (LabelsArr[i+j*9].Caption = LabelsArr[i+k*9].Caption) and (Length(LabelsArr[i+j*9].Caption)=2) then
                   begin
                      for j1:=0 to 8 do
                      begin
                          if (j1<>j) and (j1<>k) then
                          begin
                              tmptext1 := LabelsArr[i+j*9].Caption;
                              if tmptext1 <> '' then
                              begin
                                  String1 := Copy(tmptext1, 1, 1);
                                  String2 := Copy(tmptext1, 2, 1);
                                  Number1 := StrToInt(String1);
                                  Number2 := StrToInt(String2);
                                  DeleteNumberFromHint(i, j1, Number1);
                                  DeleteNumberFromHint(i, j1, Number2);
                              end;

                          end;
                      end;
                      //ShowMessage('Line ' + IntToStr(j) + ', Column ' + IntToStr(i)+ ', ' + tmptext1 + '=' + tmptext2);
                   end;
               end;
           end;
       end;
   end;

   {search for 2 equal hints in one column, eg 12 and 12
   it means that in this column other cells can not have 1 and 2}
   for j:=0 to 8 do
   begin
       for i:=0 to 8 do
       begin
           for k:=i to 8 do
           begin
               if i<>k then
               begin
                   tmptext1 :=  LabelsArr[i+j*9].Caption;
                   tmptext2 := LabelsArr[k+j*9].Caption;
                   if (LabelsArr[i+j*9].Caption = LabelsArr[k+j*9].Caption) and (Length(LabelsArr[i+j*9].Caption)=2) then
                   begin
                      for i1:=0 to 8 do
                      begin
                          if (i1<>i) and (i1<>k) then
                          begin
                              tmptext1 := LabelsArr[i+j*9].Caption;
                              if tmptext1 <> '' then
                              begin
                                  String1 := Copy(tmptext1, 1, 1);
                                  String2 := Copy(tmptext1, 2, 1);
                                  Number1 := StrToInt(String1);
                                  Number2 := StrToInt(String2);
                                  DeleteNumberFromHint(i1, j, Number1);
                                  DeleteNumberFromHint(i1, j, Number2);
                              end;
                          end;
                      end;
                      //ShowMessage('Line ' + IntToStr(j) + ', Column ' + IntToStr(i)+ ', ' + tmptext1 + '=' + tmptext2);
                   end;
               end;
           end;
       end;
   end;

   {search for 2 equal hints in one square 3x3, eg 12 and 12
   it means that in this square 3x3 other cells can not have 1 and 2}
   //Form1.Refresh;

   for i3:=0 to 2 do
   begin
       for j3:=0 to 2 do
       begin
           for i:=0 to 2 do
           begin
               for j:=0 to 2 do
               begin
                   for i1:=0 to 2 do
                       for j1:=0 to 2 do
                           if not((i1=i) and (j1=j)) then
                           begin
                               if (LabelsArr[i + i3*3 + 9*(j + j3*3)].Caption =
                                   LabelsArr[i1 + i3*3 + 9*(j1 + j3*3)].Caption)
                                   and (Length(LabelsArr[i + i3*3 + 9*(j + j3*3)].Caption)=2) then
                               begin
                                   for i2:=0 to 2 do
                                       for j2:=0 to 2 do
                                           if (not( ((i2=i)and(j2=j)) or ((i2=i1) and (j2=j1)))) then
                                           begin
                                              tmptext1 := LabelsArr[i + i3*3 + 9*(j + j3*3)].Caption;
                                              if tmptext1 <> '' then
                                              begin
                                                 String1 := Copy(tmptext1, 1, 1);
                                                 String2 := Copy(tmptext1, 2, 1);
                                                 Number1 := StrToInt(String1);
                                                 Number2 := StrToInt(String2);
                                                 DeleteNumberFromHint(i2+i3*3, j2+j3*3, Number1);
                                                 DeleteNumberFromHint(i2+i3*3, j2+j3*3, Number2);
                                                 //Form1.Refresh;
                                              end;
                                           end;
                               end;
                           end;
               end;
           end;
       end;
   end;

end;

{--------------------------------------------}
function TForm1.Cell(i:integer; j:integer) : integer;
var
    str_content : string;
begin
    str_content:=EditsArr[i+j*9].Text;
    if Length(str_content)=0 then
    begin
       Cell := 0;
       Exit;
    end;
    if (str_content[1] < '1') or
       (str_content[1] > '9') then Cell := 0
    else Cell := StrToInt(str_content);
end;

{--------------------------------------------}
procedure TForm1.Init;
begin
   EditsArr[2].Text := '9';
   EditsArr[3].Text := '4';
   EditsArr[7].Text := '8';
   EditsArr[10].Text := '2';
   EditsArr[14].Text := '5';
   EditsArr[15].Text := '3';
   EditsArr[17].Text := '7';
   EditsArr[18].Text := '4';
   EditsArr[21].Text := '3';
   EditsArr[23].Text := '6';
   EditsArr[32].Text := '7';
   EditsArr[34].Text := '3';
   EditsArr[36].Text := '8';
   EditsArr[40].Text := '3';
   EditsArr[42].Text := '1';
   EditsArr[43].Text := '9';
   EditsArr[44].Text := '6';
   EditsArr[47].Text := '6';
   EditsArr[49].Text := '5';
   EditsArr[59].Text := '8';
   EditsArr[60].Text := '9';
   EditsArr[66].Text := '5';
   EditsArr[72].Text := '7';
   EditsArr[73].Text := '6';
   EditsArr[74].Text := '4';
   EditsArr[77].Text := '3';
   EditsArr[79].Text := '5';
   EditsArr[80].Text := '8';
end;

{--------------------------------------------}
procedure TForm1.ResetWhiteEveryWhere;
var
   i, j : integer;
begin
   for i:=0 to 8 do
   begin
      for j:= 0 to 8 do
      begin
         EditsArr[i+9*j].Color := clWhite;
      end;
   end;
end;

{--------------------------------------------}
procedure TForm1.ResetMarkEveryWhere;
var
   i, j : integer;
begin
   for i:=0 to 8 do
   begin
      for j:= 0 to 8 do
      begin
         EditsArr[i+9*j].Mark := false;
      end;
   end;
end;
{--------------------------------------------}
procedure TForm1.OnEditChange(Sender: TObject);
var
   clicked_i, clicked_j : integer;
   name, hint : string;
   pcname : array[0..25] of Char;
   length : integer;
   clicked_cell : integer;
   pc_hint, pc_clicked_cell : array[0..9] of Char;
   ch_clicked_cell : Char;
begin

   {set up Assumprion flag if StartAssumption button pressed}
   name := TMyEdit (Sender).Name;
   StrPCopy(pcname, name);
   length := StrLen(pcname);
   clicked_i := Ord(pcname[length-2]) - Ord('0');
   clicked_j := Ord(pcname[length-1]) - Ord('0');
   clicked_cell := Cell(clicked_i, clicked_j);

   {if incorrect data, fix it}
   {0 assumes all symbols except 1-9}
   if clicked_cell = 0 then EditsArr[clicked_i + clicked_j*9].Text := '';

   {warning about incorrect move}
   if (clicked_cell <> 0) and (not InitInProgressFlag) then
   begin
       hint := LabelsArr[clicked_i + clicked_j*9].Caption;
       StrPCopy(pc_hint, hint);
       StrPCopy(pc_clicked_cell, IntToStr(clicked_cell));
       ch_clicked_cell := pc_clicked_cell[0];
       if StrScan(pc_hint, ch_clicked_cell) = nil then
       begin
          Application.MessageBox('Incorrect move', 'Incorrect move', MB_OK);
          EditsArr[clicked_i + clicked_j*9].Text := '';
       end;
   end;

   {assumption}
   if (clicked_cell <> 0) and (AssumptionFlag) then
   begin
       TMyEdit (Sender).Font.Color := clRed;
       TMyEdit (Sender).Assumption := true;
   end;

   {check if sudoku is solved}
   if CheckSudokuIsSolved then
   begin
       //Form2.Visible := true; showmodal
       Form2.Left := Form1.Left + Trunc(Form1.Width/2 - Form2.Width/2); 
       Form2.Top := Form1.Top + Trunc(Form1.Height/2 - Form2.Height/2); 
       Form2.ShowModal;
       //Application.MessageBox('Solved', 'Solved', MB_OK);
   end;

   InitHints;

   {update status bar}
   UpdateStatusBar;
end;


{--------------------------------------------}
procedure TForm1.OnEditClick(Sender: TObject);
var
   i, j, i1, j1, clicked_i, clicked_j : integer;
   isquare, jsquare : integer;
   name : string;
   pcname : array[0..25] of Char;
   length : integer;
   clicked_cell : integer;
begin
   name := TMyEdit (Sender).Name;
   StrPCopy(pcname, name);
   length := StrLen(pcname);
   clicked_i := Ord(pcname[length-2]) - Ord('0');
   clicked_j := Ord(pcname[length-1]) - Ord('0');
   clicked_cell := Cell(clicked_i, clicked_j);
   ResetWhiteEveryWhere;

   if clicked_cell = 0 then
   begin
      EditsArr[clicked_i+9*clicked_j].Text := '';
      Exit;
   end;

   for i:=0 to 8 do
   begin
      for j:= 0 to 8 do
      begin
          if Cell(i, j) <> 0 then
          begin
             EditsArr[i+9*j].Color := clAqua;
          end;

          if Cell(i, j) = clicked_cell then
          begin
              //fill column
              for i1 := 0 to 8 do
              begin
                  EditsArr[i1+9*j].Color := clAqua;
              end;

              //fill row
              for j1 := 0 to 8 do
              begin
                  EditsArr[i+9*j1].Color := clAqua;
              end;

              //fill square 3x3 near Cell(i, j)
              isquare := (i div 3);
              jsquare := (j div 3);
              for i1 := isquare*3 to isquare*3 + 2 do
              begin
                  for j1 := jsquare*3 to jsquare*3 + 2 do
                  begin
                      EditsArr[i1+9*j1].Color := clAqua;
                  end;
              end;
          end;
      end;
   end;
end;

{--------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
var
   NewEdit : TMyEdit;
   NewLabel : TLabel;
   i, j : integer;
begin
   InitInProgressFlag := true;
   Image1.Width := LeftMargin + RightMargin + 9 * EditXSpacing;
   Image1.Height := TopMargin + BottomMargin + 9 * EditYSpacing;
   SetLength(EditsArr, 81);
   SetLength(LabelsArr, 81);
   Randomize;
   StartTime := Time;
   for i:=0 to 8 do
   begin
      for j:= 0 to 8 do
      begin
        NewEdit := TMyEdit.Create(Form1);
        NewEdit.Parent := Form1;
        NewEdit.Text := ' ';
        //NewEdit.Text := '';
        NewEdit.Top := TopMargin + j*EditYSpacing + LabelHeight;
        NewEdit.Name := 'EditsArr' + IntToStr(i) + IntToStr(j);
        NewEdit.Left := LeftMargin + i*EditXSpacing;
        NewEdit.Width := EditWidth;
        NewEdit.Height := EditHeight;
        NewEdit.BorderStyle := bsNone;
        NewEdit.Font.Name := 'Verdana';
        NewEdit.Font.Height := EditFontHeight;
        NewEdit.MaxLength := 1;
        NewEdit.AutoSelect := false;
        NewEdit.OnClick := OnEditClick;
        NewEdit.OnChange := OnEditChange;
        NewEdit.ShowHint := true;
        NewEdit.Assumption := false;
        EditsArr[i+j*9] := NewEdit;

        NewLabel := TLabel.Create(Form1);
        NewLabel.Parent := Form1;
        //NewLabel.Caption := IntToStr(Random(10000));
        NewLabel.Caption := ' ';
        //NewLabel.Caption := '';
        NewLabel.Width := LabelWidth;
        NewLabel.Height := LabelHeight;
        NewLabel.Left := LeftMargin + i*EditXSpacing;
        NewLabel.Top := TopMargin + j*EditYSpacing;
        NewLabel.Name := 'LabelsArr' + IntToStr(i) + IntToStr(j);
        NewLabel.Font.Name := 'Verdana';
        NewLabel.Font.Height := LabelFontHeight;
        LabelsArr[i+j*9] := NewLabel;
      end;
   end;
   with Image1.Canvas do
   begin
       Brush.Color := clActiveBorder;
       FillRect(ClientRect);
       Pen.Color := clBlack;
       {vertical lines}
       MoveTo(LeftMargin + 3*EditXSpacing - 5, TopMargin);
       LineTo(LeftMargin + 3*EditXSpacing - 5, TopMargin + 9*EditYSpacing);
       MoveTo(LeftMargin + 6*EditXSpacing - 5, TopMargin);
       LineTo(LeftMargin + 6*EditXSpacing - 5, TopMargin + 9*EditYSpacing);

       {horizontal lines}
       MoveTo(LeftMargin, TopMargin + 3*EditYSpacing - 1);
       LineTo(LeftMargin + 9*EditXSpacing, TopMargin + 3*EditYSpacing-1);
       MoveTo(LeftMargin, TopMargin + 6*EditYSpacing - 1);
       LineTo(LeftMargin + 9*EditXSpacing, TopMargin + 6*EditYSpacing-1);
   end;
   AssumptionGroupBox.Left := Image1.Width;
   ClearAllFieldButton.Left := Image1.Width + Trunc((AssumptionGroupBox.Width-StartAssumptionButton.Width)/2);
   AutoSolveButton.Left := ClearAllFieldButton.Left;
   Form1.Width := AssumptionGroupBox.Left +  AssumptionGroupBox.Width + RightMargin;
   Form1.Height := Image1.Height + BottomMargin + StatusBar1.Height;
   Init;
   InitHints;
   InitInProgressFlag := false;
end;

{--------------------------------------------}
procedure TForm1.FormClick(Sender: TObject);
begin
    ResetWhiteEveryWhere;
end;

{--------------------------------------------}
procedure TForm1.FormDblClick(Sender: TObject);
begin
    ResetWhiteEveryWhere;
end;

{--------------------------------------------}
procedure TForm1.Image1Click(Sender: TObject);
begin
    ResetWhiteEveryWhere;
end;

{--------------------------------------------}
procedure TForm1.Image1DblClick(Sender: TObject);
begin
    ResetWhiteEveryWhere;
end;

{--------------------------------------------}
procedure TForm1.StartAssumptionButtonClick(Sender: TObject);
begin
    StartAssumptionButton.Enabled := false;
    CancelAssumptionButton.Enabled := true;
    ConfirmAssumptionButton.Enabled := true;
    AssumptionFlag := true;
end;

{--------------------------------------------}
procedure TForm1.CancelAssumptionButtonClick(Sender: TObject);
var
   i,j : integer;
begin
   StartAssumptionButton.Enabled := true;
   CancelAssumptionButton.Enabled := false;
   ConfirmAssumptionButton.Enabled := false;
   AssumptionFlag := false;
   for i:=0 to 8 do
   begin
      for j:= 0 to 8 do
      begin
          if EditsArr[i+9*j].Assumption then
          begin
             EditsArr[i+9*j].Assumption := false;
             EditsArr[i+9*j].Font.Color := clBlack;
             EditsArr[i+9*j].Text := ' ';
          end;
      end;
   end;
end;

{--------------------------------------------}
procedure TForm1.ConfirmAssumptionButtonClick(Sender: TObject);
var
   i,j : integer;
begin
   StartAssumptionButton.Enabled := true;
   CancelAssumptionButton.Enabled := false;
   ConfirmAssumptionButton.Enabled := false;
   AssumptionFlag := false;
   for i:=0 to 8 do
   begin
      for j:= 0 to 8 do
      begin
          if EditsArr[i+9*j].Assumption then
          begin
             EditsArr[i+9*j].Assumption := false;
             EditsArr[i+9*j].Font.Color := clBlack;
          end;
      end;
   end;
end;

{--------------------------------------------}
procedure TForm1.ClearAllFieldButtonClick(Sender: TObject);
var
   i,j : integer;
begin
   StartAssumptionButton.Enabled := true;
   CancelAssumptionButton.Enabled := false;
   ConfirmAssumptionButton.Enabled := false;
   AssumptionFlag := false;
   StartTime := Time;
   for i:=0 to 8 do
   begin
      for j:= 0 to 8 do
      begin
         EditsArr[i+9*j].Assumption := false;
         EditsArr[i+9*j].Font.Color := clBlack;
         EditsArr[i+9*j].Text := ' ';
      end;
   end;
end;

{--------------------------------------------}
function TForm1.CheckSudokuIsSolved;
var
   i, j, d, i1, j1 : integer;
   arr : array[1..9] of integer;
type
   digits = set of 1..9;
begin
   {lines}
   for i:=0 to 8 do
   begin
      for d:=1 to 9 do arr[d] := 0;

      for j:=0 to 8 do
      begin
          if (Cell(i,j)>=1) and (Cell(i,j)<=9) then Inc(arr[Cell(i,j)]);
      end;

      for d:=1 to 9 do
      begin
          if arr[d] <> 1 then
          begin
              CheckSudokuIsSolved := false;
              Exit;
          end;
      end;
   end;

   {columns}
   for j:=0 to 8 do
   begin
      for d:=1 to 9 do arr[d] := 0;

      for i:=0 to 8 do
      begin
          if (Cell(i,j)>=1) and (Cell(i,j)<=9) then Inc(arr[Cell(i,j)]);
      end;

      for d:=1 to 9 do
      begin
          if arr[d] <> 1 then
          begin
              CheckSudokuIsSolved := false;
              Exit;
          end;
      end;
   end;

   {squares 3x3}
   for i1 := 0 to 2 do
   begin
       for j1 := 0 to 2 do
       begin
           for d:=1 to 9 do arr[d] := 0;
           for i:= 0 to 2 do
           begin
               for j:= 0 to 2 do
               begin
                   if (Cell(i+i1*3, j+j1*3)>=1) and (Cell(i+i1*3, j+j1*3)<=9) then Inc(arr[Cell(i+i1*3, j+j1*3)]);
               end;
           end;

           for d:=1 to 9 do
           begin
              if arr[d] <> 1 then
              begin
                  CheckSudokuIsSolved := false;
                  Exit;
              end;
           end;
       end;
   end;

   CheckSudokuIsSolved := true;
end;

{--------------------------------------------}
procedure TForm1.UpdateStatusBar;
var
   i,j : integer;
begin
   EmptyCellsLeft := 0;
   for i:=0 to 8 do
   begin
      for j:=0 to 8 do
      begin
          if Cell(i,j)<>0 then Inc(EmptyCellsLeft);
      end;
   end;
   StatusBar1.SimpleText := 'Cells left: ' + IntToStr(81 - EmptyCellsLeft) +
   ' | Elapsed Time ' + TimeToStr(Time - StartTime);
end;

{--------------------------------------------}
procedure TForm1.Timer1Timer(Sender: TObject);
begin
    UpdateStatusBar;
end;

procedure TForm1.AutoSolveButtonClick(Sender: TObject);
var
   i, j, iList : integer;
   FoundNumbers : integer;
   NumbersArr : array [0..8] of integer;
   NumbersIndex : array [0..8] of integer;
   ListOfPoss : string;
   CurrentPoss : integer;
begin
   repeat
   FoundNumbers := 0;
   for i:=0 to 8 do
   begin
      for j:= 0 to 8 do
      begin
         if Length(LabelsArr[i+9*j].Caption) = 1 then
         begin
             EditsArr[i+9*j].Text := LabelsArr[i+9*j].Caption;
             InitHints;
             Inc(FoundNumbers);
         end;
      end;
   end;

   //check columns
   for i:=0 to 8 do
   begin
       for j:=0 to 8 do
       begin
           NumbersArr[j] := 0;
           NumbersIndex[j] := 0;
       end;

       for j:=0 to 8 do
       begin
           if Cell(i,j)=0 then
           begin
               //get list of possible numbers here
               ListOfPoss := LabelsArr[i+9*j].Caption;
               for iList:=1 to Length(ListOfPoss) do
               begin
                   CurrentPoss := StrToInt(ListOfPoss[iList]);
                   Inc(NumbersArr[CurrentPoss-1]);
                   NumbersIndex[CurrentPoss-1] := j;
               end;
           end;
       end;

       //if we found only one place for number in column
       for j:=0 to 8 do
       begin
           if NumbersArr[j] = 1 then
           begin
               EditsArr[i + NumbersIndex[j]*9].Text := IntToStr(j+1);
               InitHints;
               Inc(FoundNumbers);
           end;
       end;
   end;

   //check rows
   for j:=0 to 8 do
   begin
       for i:=0 to 8 do
       begin
           NumbersArr[i] := 0;
           NumbersIndex[i] := 0;
       end;

       for i:=0 to 8 do
       begin
           if Cell(i,j)=0 then
           begin
               //get list of possible numbers here
               ListOfPoss := LabelsArr[i+9*j].Caption;
               for iList:=1 to Length(ListOfPoss) do
               begin
                   CurrentPoss := StrToInt(ListOfPoss[iList]);
                   Inc(NumbersArr[CurrentPoss-1]);
                   NumbersIndex[CurrentPoss-1] := i;
               end;
           end;
       end;

       //if we found only one place for number in row
       for i:=0 to 8 do
       begin
           if NumbersArr[i] = 1 then
           begin
               EditsArr[NumbersIndex[i] + j*9].Text := IntToStr(i+1);
               InitHints;
               Inc(FoundNumbers);
           end;
       end;
   end;
   until FoundNumbers=0;
   //ShowMessage(IntToStr(FoundNumbers));
end;

procedure TForm1.DeleteNumberFromHint(i: integer; j:integer; value:integer);
var
    hint, newhint : string;
    k : integer;
begin
    hint := LabelsArr[i+j*9].Caption;
    newhint := '';
    for k:=1 to Length(hint) do
    begin
        if StrToInt(hint[k])<>value then
        begin
           newhint := newhint + hint[k];
        end;
    end;
    LabelsArr[i+j*9].Caption := newhint;
    EditsArr[i+j*9].Hint := newhint;
end;
procedure TForm1.TranspClick(Sender: TObject);
var
   i, j : integer;
   tmp1, tmp2 : string;
   EditsArr2 : array of string;
begin
   SetLength(EditsArr2, 81);

   for i:=0 to 8 do
   begin
       for j:=0 to 8 do
       begin
           EditsArr2[i+j*9] := EditsArr[i+j*9].Text;
       end
   end;

   ClearAllFieldButtonClick(Sender);

   for i:=0 to 8 do
   begin
       for j:=i to 8 do
       begin
           tmp1 := EditsArr2[i+j*9];
           tmp2 := EditsArr2[j+i*9];
           EditsArr2[i+j*9] := tmp2;
           EditsArr2[j+i*9] := tmp1;
       end
   end;


   for i:=0 to 8 do
   begin
       for j:=0 to 8 do
       begin
           EditsArr[i+j*9].Text := EditsArr2[i+j*9];
       end
   end;

end;

end.

